import { container } from 'tsyringe';
export * from 'tsyringe';

/**
 * A helper for register the `fn` with the `token` and wrap the `fn` with a `useResolve`.
 *
 * If a function registered with the `token` is not found, the original
 * function (`fn`) is returned.
 *
 * @see {@link https://github.com/Microsoft/tsyringe#registry}
 * @see {@link https://github.com/Microsoft/tsyringe#resolution}
 *
 * @param {string} token - name used to resolve the injection
 * @param {Function} fn - injectable function
 * @param {boolean} register - indicates if it is necessary to register the ´fn´, by default it is ´true´
 * @returns {Function} a new function wrapped with `container.resolve`
 * @example
 * ```tsx
 * import { withInjection } from '@/di/helpers'
 *
 * const useSomeFunction = () => {
 *   return 'some text'
 * }
 *
 * export default useInjection('useSomeFunction', useSomeFunction)
 * ```
 */
function withInjection(token, fn, register) {
  if (register === void 0) {
    register = true;
  }
  if (register) container.register(token, {
    useFactory: function useFactory() {
      return fn;
    }
  });
  var wrapper = function wrapper() {
    if (!container.isRegistered(token)) return fn.apply(void 0, arguments);
    var resolvedFn = container.resolve(token);
    return resolvedFn.apply(void 0, arguments);
  };
  return wrapper;
}

/**
 * Resolve a token into an instance
 *
 * @see {@link https://github.com/Microsoft/tsyringe#resolution}
 *
 * @param token — The dependency token or Class
 * @return — An instance of the dependency
 * @example
 * ´´´tsx
 *  const apiKey = useResolve<string>(API_KEY);
 *  const instance = useResolve(Service);
 * ´´´
 */
var useResolve = function useResolve(token) {
  var instance = container.resolve(token);
  return instance;
};

export { useResolve, withInjection };
//# sourceMappingURL=tsyringe-react.esm.js.map
